#include "CreateLanguageMenu.hpp"
#include "Ism.hpp"
#include "GuiManager.hpp"
#include "LanguageManager.hpp"
#include "ConfigurationManager.hpp"
#include "Font.hpp"
#include "Screen.hpp"
#include "Menu.hpp"
#include "Label.hpp"
#include "Icon.hpp"
#include "SelectLanguage.hpp"

using gui::CreateLanguageMenu;
using gui::LanguageManager;
using gui::LanguageText;
using gui::SelectLanguage;

CreateLanguageMenu::CreateLanguageMenu(BITMAP* destination)
   :
   Action(),
   destination(destination)
{
   // Lee los idiomas en los que está disponible el juego.
   this->parse(isomot::sharePath() + "text/language.xml");

   // Lee el idioma establecido en la configuración.
   ConfigurationManager configurationManager(isomot::homePath() + "configuration.xml");
   configurationManager.read();
   language = configurationManager.getLanguage();
}

CreateLanguageMenu::~CreateLanguageMenu()
{
   this->texts.clear();
}

void CreateLanguageMenu::execute()
{
   Label *label = 0;
   Screen *screen = new Screen(0, 0, this->destination);

   // Imagen de fondo.
   screen->setBackground(GuiManager::getInstance()->findImage("background"));

   // Etiqueta fija: HEAD.
   label = new Label(200, 24, "HEAD");
   label->changeFont("big", "yellow");
   screen->addWidget(label);
   // Etiqueta fija: OVER.
   label = new Label(280, 45, "OVER", "regular", "multicolor");
   screen->addWidget(label);
   // Etiqueta fija: HEELS.
   label = new Label(360, 24, "HEELS");
   label->changeFont("big", "yellow");
   screen->addWidget(label);

   // Icono: Head.
   screen->addWidget(new Icon(66, 24, GuiManager::getInstance()->findImage("head")));
   // Icono: Heels.
   screen->addWidget(new Icon(518, 24, GuiManager::getInstance()->findImage("heels")));

   // Presenta los idiomas disponibles.
   Menu *menu = new Menu(40, 120, 300, 11);
   for (std::list<LanguageText*>::iterator i = this->texts.begin(); i != this->texts.end(); ++i)
   {
      label = new Label((*i)->getText());
      label->setAction(new SelectLanguage(this->destination, (*i)->getId()));
      if ((this->language.empty() && i == this->texts.begin()) || this->language.compare((*i)->getId()) == 0)
      {
         menu->addActiveOption(label);
      }
      else
      {
         menu->addOption(label);
      }
   }
   screen->addWidget(menu);

   // Crea la cadena de responsabilidad.
   screen->setSucessor(menu);

   // Cambia la pantalla mostrada en la interfaz.
   GuiManager::getInstance()->changeScreen(screen);
}

void CreateLanguageMenu::parse(const std::string& fileName)
{
   // Carga el archivo XML especificado y almacena los datos XML en la lista.
   try
   {
      boost::property_tree::ptree xmlTree;
      boost::property_tree::xml_parser::read_xml(fileName, xmlTree);

      // Almacena todos los registros del archivo en la lista.
      auto&& textIter = xmlTree.get_child("language").equal_range("text");
      for (auto&& i = textIter.first; i != textIter.second; ++i)
      {
         LanguageText* lang = new LanguageText(i->second.get<std::string>("<xmlattr>.id"),
                                               i->second.get<unsigned int>("<xmlattr>.x"),
                                               i->second.get<unsigned int>("<xmlattr>.y"));

         auto&& propIter = i->second.equal_range("properties");
         for (auto&& j = propIter.first; j != propIter.second; ++j)
         {
            lang->addLine(j->second.get<std::string>("ustring"), std::string(), std::string());
         }

         this->texts.push_back(lang);
      }
   }
   catch (const std::runtime_error& e)
   {
      std::cerr << e.what() << std::endl;
   }
}
