#include "ConfigurationManager.hpp"

using namespace boost::property_tree;
using gui::ConfigurationManager;
using isomot::GameKey;

ConfigurationManager::ConfigurationManager(const std::string& fileName)
   :
   fileName{fileName}
{

}

void ConfigurationManager::read()
{
   // Carga el archivo XML especificado y almacena los datos XML en los diferentes atributos.
   try
   {
      ptree xmlTree;
      xml_parser::read_xml(fileName, xmlTree);

      // Almacena el idioma seleccionado.
      this->language = xmlTree.get<std::string>("configuration.language");

      // Almacena la configuración del teclado.
      this->keys[static_cast<int>(isomot::KeyNorth)] = xmlTree.get<int>("configuration.keyboard.left");
      this->keys[static_cast<int>(isomot::KeySouth)] = xmlTree.get<int>("configuration.keyboard.right");
      this->keys[static_cast<int>(isomot::KeyEast)] = xmlTree.get<int>("configuration.keyboard.up");
      this->keys[static_cast<int>(isomot::KeyWest)] = xmlTree.get<int>("configuration.keyboard.down");
      this->keys[static_cast<int>(isomot::KeyTake)] = xmlTree.get<int>("configuration.keyboard.take");
      this->keys[static_cast<int>(isomot::KeyJump)] = xmlTree.get<int>("configuration.keyboard.jump");
      this->keys[static_cast<int>(isomot::KeyShoot)] = xmlTree.get<int>("configuration.keyboard.shoot");
      this->keys[static_cast<int>(isomot::KeyTakeAndJump)] = xmlTree.get<int>("configuration.keyboard.takeandjump");
      this->keys[static_cast<int>(isomot::KeySwap)] = xmlTree.get<int>("configuration.keyboard.swap");
      this->keys[static_cast<int>(isomot::KeyHalt)] = xmlTree.get<int>("configuration.keyboard.halt");

      // Almacena los niveles de volumen.
      this->soundFxVolume = xmlTree.get<int>("configuration.volume.fx");
      this->musicVolume = xmlTree.get<int>("configuration.volume.music");
   }
   catch (const std::runtime_error& e)
   {
      std::cerr << e.what() << std::endl;
   }
}

void ConfigurationManager::write()
{
   try
   {
      ptree xmlTree;

      xmlTree.put("configuration.<xmlattr>.xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
      xmlTree.put("configuration.<xmlattr>.xsi:noNamespaceSchemaLocation", "configuration.xsd");

      xmlTree.put("configuration.language", this->language);

      xmlTree.put("configuration.keyboard.left", this->keys[isomot::KeyNorth]);
      xmlTree.put("configuration.keyboard.right", this->keys[isomot::KeySouth]);
      xmlTree.put("configuration.keyboard.up", this->keys[isomot::KeyEast]);
      xmlTree.put("configuration.keyboard.down", this->keys[isomot::KeyWest]);
      xmlTree.put("configuration.keyboard.take", this->keys[isomot::KeyTake]);
      xmlTree.put("configuration.keyboard.jump", this->keys[isomot::KeyJump]);
      xmlTree.put("configuration.keyboard.shoot", this->keys[isomot::KeyShoot]);
      xmlTree.put("configuration.keyboard.takeandjump", this->keys[isomot::KeyTakeAndJump]);
      xmlTree.put("configuration.keyboard.swap", this->keys[isomot::KeySwap]);
      xmlTree.put("configuration.keyboard.halt", this->keys[isomot::KeyHalt]);

      xmlTree.put("configuration.volume.fx", this->soundFxVolume);
      xmlTree.put("configuration.volume.music", this->musicVolume);
    
      auto settings = xml_parser::xml_writer_make_settings<ptree::key_type>(' ', 3);
      xml_parser::write_xml(fileName, xmlTree, std::locale(), settings);
   }
   catch (const std::runtime_error& e)
   {
      std::cerr << e.what() << std::endl;
   }
}
