#include "LanguageManager.hpp"
#include "Gui.hpp"

namespace gui
{

LanguageManager::LanguageManager(const std::string& fileName)
{
   this->parse(fileName);
}

LanguageManager::~LanguageManager()
{
   std::for_each(texts.begin(), texts.end(), DeleteObject());
   texts.clear();
}

LanguageText* LanguageManager::findLanguageString(const std::string& id)
{
   std::list<LanguageText*>::iterator i = std::find_if(texts.begin(), texts.end(),
                                                       [&id](const LanguageText* lang) {
                                                          return lang->getId() == id;
                                                       });
   return i != texts.end() ? *i : nullptr;
}

void LanguageManager::parse(const std::string& fileName)
{ 
   // Carga el archivo XML especificado y almacena los datos XML en la lista.
   try
   {
      boost::property_tree::ptree xmlTree;
      boost::property_tree::xml_parser::read_xml(fileName, xmlTree);
    
      // Almacena todos los registros del archivo en la lista.
      auto&& textIter = xmlTree.get_child("language").equal_range("text");
      for (auto&& i = textIter.first; i != textIter.second; ++i)
      {
         std::string id{i->second.get<std::string>("<xmlattr>.id")};
         unsigned int x{i->second.get<unsigned int>("<xmlattr>.x")};
         unsigned int y{i->second.get<unsigned int>("<xmlattr>.y")};

         LanguageText* lang = new LanguageText(id, x, y);

         auto&& propIter = i->second.equal_range("properties");
         for (auto&& j = propIter.first; j != propIter.second; ++j)
         {
            std::string font{j->second.get<std::string>("<xmlattr>.font", "")};
            std::string color{j->second.get<std::string>("<xmlattr>.color", "")};
            std::string text{j->second.get_child("ustring").data()};

            lang->addLine(text, font, color);
         }

         this->texts.push_back(lang);
      }
   }
   catch (const std::runtime_error& e)
   {
      std::cerr << e.what() << std::endl;
   }
}

LanguageText::LanguageText(const std::string& id, unsigned int x, unsigned int y)
   :
   id{id},
   x{x},
   y{y}
{

}

void LanguageText::addLine(const std::string& text)
{
   LanguageLine line;
   line.text = text;
   line.font = std::string();
   line.color = std::string();
   lines.push_back(line);
}

void LanguageText::addLine(const std::string& text, const std::string& font, const std::string& color)
{
   LanguageLine line;
   line.text = text;
   line.font = font;
   line.color = color;
   lines.push_back(line);
}

}
